package jp.mosp.time.bean.impl;

import java.util.Date;

import jp.mosp.framework.base.MospException;
import jp.mosp.framework.utils.MospUtility;
import jp.mosp.platform.bean.file.impl.BaseExportBean;
import jp.mosp.platform.dto.human.HumanDtoInterface;
import jp.mosp.time.bean.TimeMasterBeanInterface;
import jp.mosp.time.entity.ApplicationEntity;
import jp.mosp.time.entity.CutoffEntityInterface;

/**
 * エクスポート処理(勤怠管理)。<br>
 * エクスポート処理で必要となる機能を提供する。<br>
 */
public abstract class TimeBaseExportBean extends BaseExportBean {
	
	/**
	 * 勤怠関連マスタ参照処理。<br>
	 */
	protected TimeMasterBeanInterface timeMaster;
	
	
	@Override
	public void initBean() throws MospException {
		// 継承基のメソッドを実行
		super.initBean();
		// Beanの準備
		timeMaster = createBeanInstance(TimeMasterBeanInterface.class);
	}
	
	/**
	 * 対象年月及び締日から締期間初日を取得する。<br>
	 * @param cutoffCode  締日コード
	 * @param targetYear  対象年
	 * @param targetMonth 対象月
	 * @return 締期間初日
	 * @throws MospException インスタンスの取得或いはSQL実行に失敗した場合
	 */
	protected Date getCutoffFirstDate(String cutoffCode, int targetYear, int targetMonth) throws MospException {
		// 締日エンティティを取得
		CutoffEntityInterface cutoff = timeMaster.getCutoff(cutoffCode, targetYear, targetMonth);
		// 締期間初日を取得
		return cutoff.getCutoffFirstDate(targetYear, targetMonth, mospParams);
	}
	
	/**
	 * 対象年月及び締日から締期間最終日を取得する。<br>
	 * @param cutoffCode  締日コード
	 * @param targetYear  対象年
	 * @param targetMonth 対象月
	 * @return 締期間最終日
	 * @throws MospException インスタンスの取得或いはSQL実行に失敗した場合
	 */
	protected Date getCutoffLastDate(String cutoffCode, int targetYear, int targetMonth) throws MospException {
		// 締日エンティティを取得
		CutoffEntityInterface cutoff = timeMaster.getCutoff(cutoffCode, targetYear, targetMonth);
		// 締期間最終日を取得
		return cutoff.getCutoffLastDate(targetYear, targetMonth, mospParams);
	}
	
	/**
	 * 対象人事情報の対象日における締日コードが合致するかを確認する。<br>
	 * @param human      人事情報
	 * @param targetDate 対象日
	 * @param cutoffCode 締日コード
	 * @return 確認結果(true：対象人事情報の対象日における締日コードが合致する、false：合致しない)
	 * @throws MospException インスタンスの取得或いはSQL実行に失敗した場合
	 */
	protected boolean isTheCutoff(HumanDtoInterface human, Date targetDate, String cutoffCode) throws MospException {
		// 設定適用エンティティを取得
		ApplicationEntity application = timeMaster.getApplicationEntity(human, targetDate);
		// 設定適用エンティティが有効でない場合
		if (application.isValid() == false) {
			// 合致しないと判断
			return false;
		}
		// 対象人事情報の対象日における締日コードが合致するかを確認
		return MospUtility.isEqual(cutoffCode, application.getCutoffCode());
	}
	
}
